@@ -1,41 +1,49 @@
 package Net::Whois::IANA;
-
+$Net::Whois::IANA::VERSION = '0.42';
 use 5.006;
 
 use strict;
 use warnings;
 
-use Carp;
-use IO::Socket;
-
-use Net::CIDR;
+use Carp       ();
+use IO::Socket ();
+use Net::CIDR  ();
 
 use base 'Exporter';
 
+# ABSTRACT: Net::Whois::IANA - A universal WHOIS data extractor.
+
 our $WHOIS_PORT    = 43;
 our $WHOIS_TIMEOUT = 30;
+our @DEFAULT_SOURCE_ORDER = qw(arin ripe apnic lacnic afrinic);
 
-our %IANA = (
-	apnic   => [
-		[ 'whois.apnic.net',   $WHOIS_PORT, $WHOIS_TIMEOUT, \&apnic_query   ],
-	],
-	ripe    => [
-		[ 'whois.ripe.net',    $WHOIS_PORT, $WHOIS_TIMEOUT, \&ripe_query    ],
-	],
-	arin    => [
-		[ 'whois.arin.net',    $WHOIS_PORT, $WHOIS_TIMEOUT, \&arin_query    ],
-	],
-	lacnic  => [
-		[ 'whois.lacnic.net',  $WHOIS_PORT, $WHOIS_TIMEOUT, \&lacnic_query  ],
-	],
-	afrinic => [
-		[ 'whois.afrinic.net', $WHOIS_PORT, $WHOIS_TIMEOUT, \&afrinic_query ],
-	],
-);
+our %IANA;
+
+BEGIN {
+	# populate the hash at compile time
+
+	%IANA = (
+		apnic   => [
+			[ 'whois.apnic.net',   $WHOIS_PORT, $WHOIS_TIMEOUT, \&apnic_query   ],
+		],
+		ripe    => [
+			[ 'whois.ripe.net',    $WHOIS_PORT, $WHOIS_TIMEOUT, \&ripe_query    ],
+		],
+		arin    => [
+			[ 'whois.arin.net',    $WHOIS_PORT, $WHOIS_TIMEOUT, \&arin_query    ],
+		],
+		lacnic  => [
+			[ 'whois.lacnic.net',  $WHOIS_PORT, $WHOIS_TIMEOUT, \&lacnic_query  ],
+		],
+		afrinic => [
+			[ 'whois.afrinic.net', $WHOIS_PORT, $WHOIS_TIMEOUT, \&afrinic_query ],
+		],
+	);
+
+}
 
 use base 'Exporter';
 
-our $AUTOLOAD;
 our @IANA = keys %IANA;
 
 our @EXPORT = qw(
@@ -43,7 +51,21 @@ our @EXPORT = qw(
 	%IANA
 );
 
-our $VERSION = '0.41';
+BEGIN {
+	# do not use AUTOLOAD - only accept lowercase function name
+	# define accessors at compile time
+	my @accessors = qw{country netname desc status source server inetnum inet6num cidr};
+
+	foreach my $accessor ( @accessors ) {
+		no strict 'refs';
+		*$accessor = sub {
+			my ( $self ) = @_;
+			die qq[$accessor is a method call] unless ref $self;
+			return unless $self->{QUERY};
+			return $self->{QUERY}->{$accessor};
+		};
+	}
+}
 
 sub new ($) {
 
@@ -55,60 +77,37 @@ sub new ($) {
     return $self;
 }
 
-sub AUTOLOAD ($;@) {
-
-	my $self = shift;
-	my @params = @_;
-
-	my $method = $AUTOLOAD;
-	$method = lc $method;
-	my @path = split(/\:\:/, $method);
-	$method = pop @path;
-	return if $method =~ 'destroy';
-	carp "No such method or property $method\n"
-		unless exists $self->{QUERY} && exists $self->{QUERY}{$method};
-	return $self->{QUERY}{$method};
-}
-
 sub whois_connect ($;$$) {
+    my ($host, $port, $timeout) = @_;
 
-    my $host    = shift;
-	my $port;
-	my $timeout;
+    ( $host, $port, $timeout ) = @$host if ref $host;
 
-	if (ref $host && ref $host eq 'ARRAY') {
-		$port    = $host->[1] || $WHOIS_PORT;
-		$timeout = $host->[2] || $WHOIS_TIMEOUT;
-		$host    = $host->[0];
-	}
-	else {
-		$port    = shift || $WHOIS_PORT;
-		$timeout = shift || $WHOIS_TIMEOUT;
-	}
+    $port    ||= $WHOIS_PORT;
+    $timeout ||= $WHOIS_TIMEOUT;
+    #my $port    = $host_ref->[1] || $WHOIS_PORT;
+    #my $timeout = $host_ref->[2] || $WHOIS_TIMEOUT;
+    #my $host    = $host_ref->[0];
+    my $retries = 2;
+    my $sleep   = 2;
 
-	my $retries = 2;
-	my $sleep   = 1;
-	my $r = 0;
-	my $sock;
-
-	do {
-		if ($r) {
-			carp "Cannot connect to $host at port $port";
-			carp $@;
-			sleep $sleep;
-		}
-		$sock = IO::Socket::INET->new(
-			PeerAddr => $host,
-			PeerPort => $port,
-			Timeout  => $timeout,
-		);
-		$r++;
-	} until ($sock || $r == $retries);
-
-    return $sock || 0;
+    for ( 0 .. $retries ) {
+        if (
+            my $sock = IO::Socket::INET->new(
+                PeerAddr => $host,
+                PeerPort => $port,
+                Timeout  => $timeout,
+            )
+          ) {
+            return $sock;
+        }
+        Carp::carp "Cannot connect to $host at port $port";
+        Carp::carp $@;
+        sleep $sleep;
+    }
+    return 0;
 }
 
-sub is_valid_ip ($) {
+sub is_valid_ipv4 ($) {
 
 	my $ip = shift;
 
@@ -117,6 +116,49 @@ sub is_valid_ip ($) {
 		&& (($1+0)|($2+0)|($3+0)|($4+0)) < 0x100;
 }
 
+sub is_valid_ipv6 {
+    my ($ip) = @_;
+    return if $ip =~ /^:[^:]/ || $ip =~ /[^:]:$/;    # Can't have single : on front or back
+
+    my @seg = split /:/, $ip, -1;                    # -1 to keep trailing empty fields
+                                                     # Clean up leading/trailing double colon effects.
+    shift @seg if $seg[0] eq '';
+    pop @seg   if $seg[-1] eq '';
+
+    my $max = 8;
+    if ( $seg[-1] =~ tr/.// ) {
+        return unless is_valid_ipv4( pop @seg );
+        $max -= 2;
+    }
+
+    my $cmp;
+    for my $seg (@seg) {
+        if ( $seg eq '' ) {
+
+            # Only one compression segment allowed.
+            return if $cmp;
+            ++$cmp;
+            next;
+        }
+        return if $seg =~ /[^0-9a-fA-F]/;
+        return if length $seg == 0 || length $seg > 4;
+    }
+    if ($cmp) {
+
+        # If compressed, we need fewer than $max segments, but at least 1
+        return ( @seg && @seg < $max ) && 1;    # true returned as 1
+    }
+
+    # Not compressed, all segments need to be there.
+    return $max == @seg;
+}
+
+# Is valid IP v4 or IP v6 address.
+sub is_valid_ip ($) {
+    my($ip) = @_;
+    return defined $ip && $ip =~ tr/:// ? is_valid_ipv6($ip) : is_valid_ipv4($ip);
+}
+
 sub set_source ($$) {
 
 	my $self   = shift;
@@ -179,21 +221,17 @@ Custom sources must be of form:
 }
 
 sub source_connect ($$) {
+    my ( $self, $source_name ) = @_;
 
-	my $self = shift;
-	my $source_name = shift;
-	my $i = 0;
-	my $sock;
-	do {
-		$sock = whois_connect($self->{source}{$source_name}[$i]);
-		$self->{query_sub} =
-			ref $self->{source}{$source_name}[$i][3] &&
-			ref $self->{source}{$source_name}[$i][3] eq 'CODE' ?
-				$self->{source}{$source_name}[$i][3] : \&default_query;
-		$self->{whois_host} = $self->{source}{$source_name}[$i][0];
-		$i++;
-	} until ($sock || !defined $self->{source}{$source_name}[$i]);
-	return $sock;
+    foreach my $server_ref ( @{ $self->{source}{$source_name} } ) {
+        if ( my $sock = whois_connect($server_ref) ) {
+            my ( $whois_host, $whois_port, $whois_timeout, $query_code ) = @{$server_ref};
+            $self->{query_sub} = $query_code && ref $query_code eq 'CODE' ? $query_code : \&default_query;
+            $self->{whois_host} = $whois_host;
+            return $sock;
+        }
+    }
+    return undef;
 }
 
 sub post_process_query (%) {
@@ -228,17 +266,16 @@ sub whois_query ($%) {
 
 	my $self = shift;
 	my %params = @_;
-
 	$self->init_query(%params);
-	my @source_names = keys %{$self->{source}};
+	my @source_names = @DEFAULT_SOURCE_ORDER;
     $self->{QUERY} = {};
     for my $source_name (@source_names) {
 		print STDERR "Querying $source_name ...\n" if $params{-debug};
 		my $sock = $self->source_connect($source_name) ||
-			carp "Connection failed to $source_name." && next;
+			Carp::carp "Connection failed to $source_name." && next;
 		my %query = $self->{query_sub}($sock, $params{-ip});
 		next if (! keys %query);
-		carp "Warning: permission denied at $source_name server $self->{whois_host}\n" and next
+		Carp::carp "Warning: permission denied at $source_name server $self->{whois_host}\n" and next
 			if $query{permission} eq 'denied';
 		$query{server} = uc $source_name;
 		$self->{QUERY} = {post_process_query(%query)};
@@ -265,10 +302,9 @@ sub ripe_read_query ($$) {
 		close $sock and return (permission => 'denied') if /ERROR:201/;
 		next if (/^(\%|\#)/ || !/\:/);
 		s/\s+$//;
-		my ($field,$value) = split(/:/);
+		my ($field,$value) = split(/:/, $_, 2);
 		$value =~ s/^\s+//;
-		$query{$field} .= $value;
-		last if (/^route/);
+    $query{lc($field)} .= ( $query{lc($field)} ?  ' ' : '') . $value;
     }
     close $sock;
 	return %query;
@@ -294,9 +330,12 @@ sub ripe_process_query (%) {
 	) {
 		return ();
     }
+    elsif ( !$query{inet6num} && !$query{inetnum} ) {
+        return ();
+    }
     else {
 		$query{permission} = 'allowed';
-        $query{cidr} = [ Net::CIDR::range2cidr($query{inetnum}) ];
+        $query{cidr} = [Net::CIDR::range2cidr(uc($query{inet6num} || $query{inetnum}))];
     }
     return %query;
 }
@@ -314,39 +353,57 @@ sub ripe_query ($$) {
 sub apnic_read_query ($$) {
 
     my $sock = shift;
-    my $ip = shift;
+    my $ip   = shift;
 
-    my %query = (fullinfo => '');
-	my %tmp;
+    my %query = ( fullinfo => '' );
+    my %tmp;
     print $sock "-r $ip\n";
+    my $skip_block = 0;
     while (<$sock>) {
-		$query{fullinfo} .= $_;
-		close $sock and	return (permission => 'denied') if /^\%201/;
-		next if (/^\%/ || !/\:/);
-		s/\s+$//;
-		my ($field,$value) = split(/:/);
-		$value =~ s/^\s+//;
-		if ($field eq 'inetnum') {
-			%tmp = %query;
-			%query = ();
-			$query{fullinfo} = $tmp{fullinfo};
-		}
-		$query{$field} .= $value;
+        $query{fullinfo} .= $_;
+        close $sock and return ( permission => 'denied' ) if /^\%201/;
+        if (m{^\%}) {
+
+            # Always skip 0.0.0.0 data
+            # It looks like:
+            # % Information related to '0.0.0.0 - 255.255.255.255'
+            if (m{^\%.*0\.0\.0\.0\s+}) {
+                $skip_block = 1;
+                next;
+            }
+            $skip_block = 0;
+            next;
+        }
+        next if $skip_block;
+        next if ( !/\:/ );
+        s/\s+$//;
+        my ( $field, $value ) = split( /:/, $_, 2 );
+        $value =~ s/^\s+//;
+        if ( $field =~ /^inet6?num$/ ) {
+            next if $value =~ m{0\.0\.0\.0\s+};
+            %tmp             = %query;
+            %query           = ();
+            $query{fullinfo} = $tmp{fullinfo};
+        }
+        my $lc_field = lc($field);
+        next if $lc_field eq 'country' && defined $query{ $lc_field };
+        $query{ $lc_field } .= ( $query{ $lc_field } ? ' ' : '' ) . $value;
     }
     close $sock;
-    for (keys %tmp) {
-		$query{$_} = $tmp{$_} if ! defined $query{$_};
+    for ( keys %tmp ) {
+        $query{$_} = $tmp{$_} if !defined $query{$_};
     }
-	return %query;
+    return %query;
 }
 
+
 sub apnic_process_query (%) {
 
 	my %query = @_;
     if (
 		(
 			defined $query{remarks} &&
-			$query{remarks} =~ /address range is not administered by APNIC/
+			$query{remarks} =~ /address range is not administered by APNIC|This network in not allocated/
 		) || (
 			defined $query{descr} &&
 			$query{descr} =~ /not allocated to|by APNIC|placeholder reference/i
@@ -354,9 +411,12 @@ sub apnic_process_query (%) {
 	) {
 		return ();
     }
+    elsif ( !$query{inet6num} && !$query{inetnum} ) {
+        return ();
+    }
     else {
     	$query{permission} = 'allowed';
-		$query{cidr} = [Net::CIDR::range2cidr($query{inetnum})];
+        $query{cidr} = [Net::CIDR::range2cidr(uc($query{inet6num} || $query{inetnum}))];
     }
     return %query;
 }
@@ -385,7 +445,7 @@ sub arin_read_query ($$) {
 		return () if /no match found for/i;
 		next if (/^\#/ || !/\:/);
 		s/\s+$//;
-		my ($field,$value) = split(/:/);
+		my ($field,$value) = split(/:/, $_, 2);
 		$value =~ s/^\s+//;
 		if ($field eq 'OrgName' ||
 				$field eq 'CustName') {
@@ -393,7 +453,7 @@ sub arin_read_query ($$) {
 			%query = ();
 			$query{fullinfo} = $tmp{fullinfo};
 		}
-		$query{lc($field)} .= $value;
+    $query{lc($field)} .= ( $query{lc($field)} ?  ' ' : '') . $value;
     }
     close $sock;
     $query{orgname} = $query{custname} if defined $query{custname};
@@ -404,16 +464,8 @@ sub arin_read_query ($$) {
 }
 
 sub arin_process_query (%) {
-
 	my %query = @_;
-
-    return () unless
-		$query{country} or
-		$query{nettype} !~ /allocated to/i or
-			$query{comment} &&
-			$query{comment} =~ /This IP address range is not registered in the ARIN/ or
-			$query{orgid} &&
-			$query{orgid} =~ /RIPE|LACNIC|APNIC|AFRINIC/;
+    return () if $query{orgid} && $query{orgid} =~ /^\s*RIPE|LACNIC|APNIC|AFRINIC\s*$/;
 
 	$query{permission} = 'allowed';
 	$query{descr}   = $query{orgname};
@@ -460,7 +512,7 @@ sub lacnic_read_query ($$) {
 		}
 		next if (/^\%/ || !/\:/);
 		s/\s+$//;
-		my ($field,$value) = split(/:/);
+		my ($field,$value) = split(/:/, $_, 2);
 		$value =~ s/^\s+//;
 		next if $field eq 'country' && $query{country};
 		$query{lc($field)} .= ( $query{lc($field)} ?  ' ' : '') . $value;
@@ -503,25 +555,7 @@ sub lacnic_query ($$) {
 	return lacnic_process_query(%query);
 }
 
-sub afrinic_read_query ($$) {
-
-    my $sock = shift;
-    my $ip = shift;
-
-    my %query = (fullinfo => '');
-    print $sock "-r $ip\n";
-    while (<$sock>) {
-        $query{fullinfo} .= $_;
-        close $sock and return (permission => 'denied') if /^\%201/;
-        next if (/^\%/ || !/\:/);
-        s/\s+$//;
-        my ($field,$value) = split(/:/);
-        $value =~ s/^\s+//;
-        $query{$field} .= $value;
-    }
-    close $sock;
-	return %query;
-}
+*afrinic_read_query = *apnic_read_query;
 
 sub afrinic_process_query (%) {
 
@@ -533,8 +567,13 @@ sub afrinic_process_query (%) {
 			or
 		defined $query{descr} &&
 		$query{descr} =~ /Here for in-addr\.arpa authentication/;
+
+    if ( !$query{inet6num} && !$query{inetnum} ) {
+        return ();
+    }
+
 	$query{permission} = 'allowed';
-	$query{cidr} = [ Net::CIDR::range2cidr($query{inetnum}) ];
+    $query{cidr} = [Net::CIDR::range2cidr(uc($query{inet6num} || $query{inetnum}))];
     return %query;
 }
 
@@ -568,15 +607,23 @@ sub is_mine ($$;@) {
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-Net::Whois::IANA - A universal WHOIS data extractor.
+Net::Whois::IANA - Net::Whois::IANA - A universal WHOIS data extractor.
+
+=head1 VERSION
+
+version 0.42
 
 =head1 SYNOPSIS
 
   use Net::Whois::IANA;
   my $ip = '132.66.16.2';
-  my $iana = new Net::Whois::IANA;
+  my $iana = Net::Whois::IANA->new;
   $iana->whois_query(-ip=>$ip);
   print "Country: " , $iana->country()            , "\n";
   print "Netname: " , $iana->netname()            , "\n";
@@ -587,19 +634,6 @@ Net::Whois::IANA - A universal WHOIS data extractor.
   print "Inetnum: " , $iana->inetnum()            , "\n";
   print "CIDR: "    , join(",", $iana->cidr())    , "\n";
 
-
-=head1 ABSTRACT
-
-  This is a simple module to extract the descriptive whois
-information about various IPs as they are stored in the four
-regional whois registries of IANA - RIPE (Europe, Middle East)
-APNIC (Asia/Pacific), ARIN (North America), AFRINIC (Africa) 
-and LACNIC (Latin American & Caribbean).
-
-  It is designed to serve statistical harvesters of various
-access logs and likewise, therefore it only collects partial
-and [rarely] unprecise information.
-
 =head1 DESCRIPTION
 
   Various Net::Whois and IP:: modules have been created.
@@ -706,6 +740,22 @@ being exported.
   candidate. This is not a very reliable thing, but sometimes it proves
   useful.
 
+=head1 NAME
+
+Net::Whois::IANA - A universal WHOIS data extractor.
+
+=head1 ABSTRACT
+
+  This is a simple module to extract the descriptive whois
+information about various IPs as they are stored in the four
+regional whois registries of IANA - RIPE (Europe, Middle East)
+APNIC (Asia/Pacific), ARIN (North America), AFRINIC (Africa)
+and LACNIC (Latin American & Caribbean).
+
+  It is designed to serve statistical harvesters of various
+access logs and likewise, therefore it only collects partial
+and [rarely] unprecise information.
+
 =head1 BUGS
 
   As stated many times before, this module is not completely
@@ -742,4 +792,15 @@ Copyright 2003-2011 Bolet Consulting <bolet@parparov.com> and Roman M. Parparov
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
+=head1 AUTHOR
+
+Nicolas R <atoomic@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2003 by Bolet Consulting.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
